package com.qmth.distributed.print.business.templete.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.distributed.print.business.bean.dto.ExaminationExportDto;
import com.qmth.distributed.print.business.bean.dto.FieldsDto;
import com.qmth.distributed.print.business.bean.dto.PdfDto;
import com.qmth.distributed.print.business.entity.*;
import com.qmth.distributed.print.business.enums.ExamStatusEnum;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.enums.UploadFileEnum;
import com.qmth.distributed.print.business.service.*;
import com.qmth.distributed.print.business.templete.service.TaskLogicService;
import com.qmth.distributed.print.business.util.ConvertUtil;
import com.qmth.distributed.print.business.util.CreatePdfUtil;
import com.qmth.distributed.print.business.util.ExcelUtil;
import com.qmth.distributed.print.business.util.OssUtil;
import com.qmth.distributed.print.common.contant.SystemConstant;
import com.qmth.distributed.print.common.enums.ExceptionResultEnum;
import com.qmth.distributed.print.common.util.ResultUtil;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.*;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 任务处理逻辑impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/29
 */
@Service
public class TaskLogicServiceImpl implements TaskLogicService {
    private final static Logger log = LoggerFactory.getLogger(TaskLogicServiceImpl.class);

    @Resource
    CommonService commonService;

    @Resource
    ExamDetailService detailService;

    @Resource
    ExamDetailCourseService detailCourseService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    ExamTaskDetailService examTaskDetailService;

    @Resource
    ExamCardService examCardService;

    @Resource
    ExamCardDetailService examCardDetailService;

    @Resource
    ExamStudentService examStudentService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    ExamDetailService examDetailService;

    @Resource
    OssUtil ossUtil;

    @Resource
    CacheService cacheService;

    @Resource
    BasicCourseService basicCourseService;

    @Resource
    CreatePdfUtil createPdfUtil;

    /**
     * 创建pdf逻辑
     *
     * @param map
     * @return
     */
    @Override
    @Transactional
    public Map<String, Object> executeCreatePdfLogic(Map<String, Object> map) throws IOException {
        List<BasicAttachment> basicAttachmentList = null;
        try {
            TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
            SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
            Long examDetailId = Objects.nonNull(map.get("examDetailId")) ? (Long) map.get("examDetailId") : null;
            Long schoolId = (Long) map.get("schoolId");

            //这里为保存附件html的逻辑
            //查询题卡详情是否有附件id，有的话则把以前的附件删除
            Set<Long> attachmentIds = null;
            Set<File> ftlList = null;

            //查询printPlan
            ExamPrintPlan examPrintPlan = examPrintPlanService.getById(tbTask.getPrintPlanId());
            if (Objects.isNull(examPrintPlan)) {
                throw ExceptionResultEnum.ERROR.exception("印刷计划为空");
            }

            BasicSchool basicSchool = cacheService.schoolCache(examPrintPlan.getSchoolId());

            //查询examDetail
            QueryWrapper<ExamDetail> examDetailQueryWrapper = new QueryWrapper<>();
            examDetailQueryWrapper.lambda().eq(ExamDetail::getSchoolId, sysUser.getSchoolId())
                    .eq(ExamDetail::getPrintPlanId, tbTask.getPrintPlanId());
            if (Objects.nonNull(examDetailId)) {
                examDetailQueryWrapper.lambda().eq(ExamDetail::getId, examDetailId);
            }
            List<ExamDetail> examDetailList = detailService.list(examDetailQueryWrapper);
            if (Objects.isNull(examDetailList) || examDetailList.size() == 0) {
                throw ExceptionResultEnum.ERROR.exception("考务计划为空");
            }
            attachmentIds = Objects.isNull(attachmentIds) ? attachmentIds = new HashSet<>() : attachmentIds;
            ftlList = Objects.isNull(ftlList) ? ftlList = new HashSet<>() : ftlList;
            for (ExamDetail examDetail : examDetailList) {
                //查询examDetailCourse
                QueryWrapper<ExamDetailCourse> examDetailCourseQueryWrapper = new QueryWrapper<>();
                examDetailCourseQueryWrapper.lambda().eq(ExamDetailCourse::getExamDetailId, examDetail.getId());
                List<ExamDetailCourse> examDetailCourseList = detailCourseService.list(examDetailCourseQueryWrapper);

                List<PdfDto> variablePdfList = new ArrayList<>();//变量印品(签到表、卷袋贴)
                List<PdfDto> ordinaryPdfList = new ArrayList<>();//普通印品(登记表)
                List<PdfDto> paperPdfList = new ArrayList<>();//所有试卷
                List<PdfDto> examStudentPdfList = new ArrayList<>();//所有题卡
                List<PdfDto> backupPaperPdfList = new ArrayList<>();//备份试卷
                List<PdfDto> cardPdfList = new ArrayList<>();//备份题卡

                if (Objects.nonNull(examPrintPlan.getOrdinaryContent())) {
                    //获取普通印品
                    JSONArray jsonArrayOrdinary = JSONArray.parseArray(examPrintPlan.getOrdinaryContent());
                    for (int i = 0; i < jsonArrayOrdinary.size(); i++) {
                        JSONObject jsonObjectOrdinary = jsonArrayOrdinary.getJSONObject(i);
                        Long attachmentId = Long.parseLong((String) jsonObjectOrdinary.get("attachmentId"));
                        BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                        createPdfUtil.createCheckIn(basicAttachment, ordinaryPdfList, (Integer) jsonObjectOrdinary.get("backupCount"));
                    }
                }

                for (ExamDetailCourse examDetailCourse : examDetailCourseList) {
                    List<PdfDto> paperPdfTempList = new ArrayList<>();//所有试卷
                    List<PdfDto> examStudentTempPdfList = new ArrayList<>();//所有题卡
                    List<PdfDto> backupPaperTempPdfList = new ArrayList<>();//备份试卷
                    List<PdfDto> cardPdfTempList = new ArrayList<>();//备份题卡

                    //查询试卷
                    QueryWrapper<ExamTask> examTaskQueryWrapper = new QueryWrapper<>();
                    examTaskQueryWrapper.lambda().eq(ExamTask::getSchoolId, sysUser.getSchoolId())
                            .eq(ExamTask::getCourseCode, examDetailCourse.getCourseCode())
                            .eq(ExamTask::getCourseName, examDetailCourse.getCourseName())
                            .eq(ExamTask::getPaperNumber, examDetailCourse.getPaperNumber())
                            .eq(ExamTask::getEnable, true)
                            .eq(ExamTask::getStatus, ExamStatusEnum.FINISH);
                    List<ExamTask> examTaskList = examTaskService.list(examTaskQueryWrapper);
                    if (Objects.isNull(examTaskList) || examTaskList.size() == 0) {
                        throw ExceptionResultEnum.ERROR.exception("命题任务为空");
                    }
                    Set<Long> examTaskIds = examTaskList.stream().map(s -> s.getId()).collect(Collectors.toSet());

                    QueryWrapper<ExamTaskDetail> examTaskDetailQueryWrapper = new QueryWrapper<>();
                    examTaskDetailQueryWrapper.lambda().in(ExamTaskDetail::getExamTaskId, examTaskIds)
                            .eq(ExamTaskDetail::getEnable, true);
                    List<ExamTaskDetail> examTaskDetailList = examTaskDetailService.list(examTaskDetailQueryWrapper);
                    JSONObject jsonObject = new JSONObject();
                    JSONArray jsonArray = new JSONArray();
                    for (ExamTaskDetail examTaskDetail : examTaskDetailList) {
                        //查询题卡
                        ExamCard examCard = examCardService.getById(examTaskDetail.getCardId());
                        if (Objects.isNull(examCard)) {
                            throw ExceptionResultEnum.ERROR.exception("题卡为空");
                        }
                        QueryWrapper<ExamCardDetail> examCardDetailQueryWrapper = new QueryWrapper<>();
                        examCardDetailQueryWrapper.lambda().eq(ExamCardDetail::getCardId, examCard.getId());
                        List<ExamCardDetail> examCardDetailList = examCardDetailService.list(examCardDetailQueryWrapper);

                        //抽取卷型
                        String paperType = createPdfUtil.getPaperType(examPrintPlan, examTaskDetail);

                        //获取试卷pdf
                        createPdfUtil.getPaperPdf(examTaskDetail, examPrintPlan.getBackupCount(), paperPdfTempList, backupPaperTempPdfList);
                        paperPdfList.addAll(paperPdfTempList);
                        backupPaperPdfList.addAll(backupPaperTempPdfList);

                        basicAttachmentList = Objects.isNull(basicAttachmentList) ? basicAttachmentList = new ArrayList<>() : basicAttachmentList;
                        for (ExamCardDetail examCardDetail : examCardDetailList) {
                            createPdfUtil.getCardAttachmentId(examCardDetail, attachmentIds);

                            //把模板页面上的 ${} 替换成实际内容
                            String cardContent = createPdfUtil.replaceHtmlTemplete(examCardDetail);
                            String studentContent = cardContent;

                            for (int i = 1; i <= examPrintPlan.getBackupCount(); i++) {
                                basicAttachmentList.add(createPdfUtil.cardHtml(cardContent, paperType, examDetailCourse, examCard, jsonArray, sysUser.getId(), cardPdfTempList));
                            }
                            cardPdfList.addAll(cardPdfTempList);

                            //查询考生
                            QueryWrapper<ExamStudent> examStudentQueryWrapper = new QueryWrapper<>();
                            examStudentQueryWrapper.lambda().eq(ExamStudent::getSchoolId, sysUser.getSchoolId())
                                    .eq(ExamStudent::getExamDetailCourseId, examDetailCourse.getId());
                            List<ExamStudent> examStudentList = examStudentService.list(examStudentQueryWrapper);

                            if (Objects.nonNull(examPrintPlan.getVariableContent())) {
                                //获取变量印品
                                JSONArray jsonArrayVariable = JSONArray.parseArray(examPrintPlan.getVariableContent());
                                for (int i = 0; i < jsonArrayVariable.size(); i++) {
                                    JSONObject jsonObjectVariable = jsonArrayVariable.getJSONObject(i);
                                    String type = (String) jsonObjectVariable.get("type");
                                    Long attachmentId = Long.parseLong((String) jsonObjectVariable.get("attachmentId"));
                                    BasicAttachment basicAttachment = basicAttachmentService.getById(attachmentId);
                                    ftlList.add(commonService.getFile(basicAttachment.getPath(), false));
                                    if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "SIGN")) {//签到表
                                        createPdfUtil.createSignBook(basicAttachment, basicSchool.getName(), examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                    } else if (Objects.nonNull(type) && Objects.equals(type.toUpperCase(), "PACKAGE")) {//卷袋贴
                                        createPdfUtil.createPaperPackage(basicAttachment, basicSchool.getName(), examDetailCourse, examStudentList, variablePdfList, (Integer) jsonObjectVariable.get("backupCount"));
                                    }
                                }
                            }

                            for (ExamStudent t : examStudentList) {
                                basicAttachmentList.add(createPdfUtil.examStudentHtml(attachmentIds, studentContent, t, paperType, examCard, sysUser.getId(), examStudentTempPdfList));
                            }
                            examStudentPdfList.addAll(examStudentTempPdfList);
                            examStudentService.saveOrUpdateBatch(examStudentList);
                            jsonObject.put("card", jsonArray);
                            examCardDetail.setAttachmentId(jsonObject.toJSONString());
                        }
                        examCardDetailService.saveOrUpdateBatch(examCardDetailList);
                    }
                    examTaskDetailService.saveOrUpdateBatch(examTaskDetailList);

                    Integer paperPageA3 = 0;
                    Integer cardPageA3 = 0;
                    paperPageA3 = paperPageA3 + paperPdfTempList.stream().mapToInt(PdfDto::getPageCount).sum() + backupPaperTempPdfList.stream().mapToInt(PdfDto::getPageCount).sum();
                    cardPageA3 = cardPageA3 + examStudentTempPdfList.stream().mapToInt(PdfDto::getPageCount).sum() + cardPdfTempList.stream().mapToInt(PdfDto::getPageCount).sum();
                    examDetailCourse.setPaperPagesA3(paperPageA3);
                    examDetailCourse.setCardPagesA3(cardPageA3);
                }
                detailCourseService.saveOrUpdateBatch(examDetailCourseList);
                //合并pdf
                basicAttachmentList.add(createPdfUtil.mergePdf(tbTask, examDetail, sysUser.getId(), schoolId, variablePdfList, ordinaryPdfList, paperPdfList, examStudentPdfList, backupPaperPdfList, cardPdfList));
            }
            map.computeIfAbsent("size", v -> examDetailList.size());
            //最后一步删除附件
            createPdfUtil.deleteAttachment(attachmentIds, ftlList);
        } catch (Exception e) {
            log.error("请求出错", e);
            basicAttachmentService.batchDeleteAttachment(basicAttachmentList);
            if (e instanceof ApiException) {
                ResultUtil.error((ApiException) e, e.getMessage());
            } else {
                ResultUtil.error(e.getMessage());
            }
        }
        return map;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeExaminationLogic(Map<String, Object> map) throws Exception {
        List<ExaminationExportDto> examinationExportDtoList = examDetailService.findExaminationExportDtoDatasource(map);

        System.out.println("examinationResultList = " + JSON.toJSONString(examinationExportDtoList));

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        ExcelUtil.excelMake(ExaminationExportDto.class, examinationExportDtoList, outputStream);
        InputStream in = new ByteArrayInputStream(outputStream.toByteArray());

//        UploadFileEnum.FILE/
        LocalDateTime nowTime = LocalDateTime.now();
        StringJoiner stringJoiner = new StringJoiner("");
        stringJoiner.add(UploadFileEnum.FILE.getTitle()).add(File.separator);

        stringJoiner.add(String.valueOf(nowTime.getYear())).add(File.separator)
                .add(String.format("%02d", nowTime.getMonthValue())).add(File.separator)
                .add(String.format("%02d", nowTime.getDayOfMonth()));

        JSONObject jsonObject = new JSONObject();
        stringJoiner.add(File.separator).add(SystemConstant.getUuid()).add(".").add(SystemConstant.XLSX);

        String dirName = stringJoiner.toString().replaceAll("\\\\", "/");

        ossUtil.ossUpload(dirName, in, null);
        jsonObject.put(SystemConstant.TYPE, SystemConstant.OSS);
        jsonObject.put(SystemConstant.PATH, dirName);
        jsonObject.put(SystemConstant.UPLOAD_TYPE, UploadFileEnum.FILE);
        map.put("path", jsonObject.toString());
        return map;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeImportExaminationLogic(Map<String, Object> map) throws Exception {
        TBTask tbTask = (TBTask) map.get(SystemConstant.TASK);
        InputStream inputStream = (InputStream) map.get("inputStream");
        Long printPlanId = tbTask.getPrintPlanId();
        Long userId = tbTask.getCreateId();
        ExamPrintPlan examPrintPlan = examPrintPlanService.getById(printPlanId);

        if (examPrintPlan == null) {
            throw ExceptionResultEnum.ERROR.exception("印刷计划不存在printPlanId ： " + printPlanId);
        }

        PrintPlanStatusEnum printPlanStatus = examPrintPlan.getStatus();
        if (PrintPlanStatusEnum.NEW != printPlanStatus && PrintPlanStatusEnum.READY != printPlanStatus) {
            throw ExceptionResultEnum.ERROR.exception(printPlanStatus + "状态下的印刷计划不允许导入");
        }

        String printPlanName = examPrintPlan.getName();
        Long schoolId = tbTask.getSchoolId();
        // 该学校有效考务数据
        List<FieldsDto> fieldsDtoList = examDetailService.findExaminationFields(schoolId);
        if (fieldsDtoList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("该学校没有设置考务数据");
        }
        String importFilePath = tbTask.getImportFilePath();
        Map importFilePathMap = JSONObject.parseObject(importFilePath);
        String path = String.valueOf(importFilePathMap.get("path"));
        System.out.println("path = " + path);
        Workbook workbook;
        if (path.endsWith(SystemConstant.XLSX)) {
            workbook = new XSSFWorkbook(inputStream);
        } else if (path.endsWith("xls")) {
            workbook = new HSSFWorkbook(inputStream);
        } else {
            throw ExceptionResultEnum.ERROR.exception("文件格式异常");
        }
        // 读取第一个工作表
        Sheet sheet = workbook.getSheetAt(0);
        // 获取sheet行数
        int totalRows = sheet.getPhysicalNumberOfRows();
        // 获取sheet列数
        int totalCells = 0;
        if (totalRows > 1 && sheet.getRow(0) != null) {
            totalCells = sheet.getRow(0).getPhysicalNumberOfCells();
        }
        System.out.println("行 ： " + totalRows + "\n列 ： " + totalCells);
        Row head = sheet.getRow(0);
        List<String> headList = new ArrayList<>();
        // 将必填字段匹配excel解析的表头索引
        for (int i = 0; i < totalCells; i++) {
            String cellValue = head.getCell(i).getStringCellValue();
            for (FieldsDto fieldsDto : fieldsDtoList) {
                if (cellValue.equals(fieldsDto.getName())) {
                    fieldsDto.setIndex(i);
                }
            }
            headList.add(cellValue);
        }

        // 搜索所有有效字段 excel中的表头是否包含
        for (FieldsDto fieldsDto : fieldsDtoList) {
            if (!headList.contains(fieldsDto.getName())) {
                System.out.println("exception " + fieldsDto.getName());
                throw ExceptionResultEnum.ERROR.exception("学校考务必填字段 ：'" + fieldsDto.getName() + "' 不存在");
            }
        }

        System.out.println("headList : " + headList);
        System.out.println("fieldsDtoList : " + JSONObject.toJSONString(fieldsDtoList));

        List<Map<String, Object>> dataList = new ArrayList<>();

        for (int r = 1; r < totalRows; r++) {
            Row row = sheet.getRow(r);

            String studentCode = null;
            String studentName = null;
            String courseCode = null;
            String courseName = null;
            String examPlace = null;
            String examRoom = null;
            String examDate = null;
            String examTime = null;
            String paperNumber = null;
            List<FieldsDto> secondaryFieldList = new ArrayList<>();
            for (FieldsDto fieldsDto : fieldsDtoList) {
                String name = fieldsDto.getName();
                String code = fieldsDto.getCode();
                int index = fieldsDto.getIndex();
                String level = fieldsDto.getLevel();
                if ("学号".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    studentCode = row.getCell(index).getStringCellValue();
                } else if ("姓名".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    studentName = row.getCell(index).getStringCellValue();
                } else if ("课程代码".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    courseCode = row.getCell(index).getStringCellValue();
                } else if ("课程名称".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    courseName = row.getCell(index).getStringCellValue();
                } else if ("考点".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examPlace = row.getCell(index).getStringCellValue();
                } else if ("考场".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examRoom = row.getCell(index).getStringCellValue();
                } else if ("考试日期".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examDate = row.getCell(index).getStringCellValue();
                } else if ("考试时间".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    examTime = row.getCell(index).getStringCellValue();
                } else if ("试卷编号".equals(name)) {
                    row.getCell(index).setCellType(CellType.STRING);
                    paperNumber = row.getCell(index).getStringCellValue();
                } else {
                    if ("primary".equals(level)) {
                        throw ExceptionResultEnum.ERROR.exception("有数据库不需要的必选字段 ： " + name);
                    }
                    if ("secondary".equals(level)) {
                        // 备选字段处理
                        FieldsDto secondaryField = new FieldsDto();
                        secondaryField.setLevel(level);
                        secondaryField.setCode(code);
                        secondaryField.setName(name);
                        secondaryField.setEnable(true);
                        secondaryField.setValue(row.getCell(index).getStringCellValue());
                        secondaryFieldList.add(secondaryField);
                    }
                }
            }
            // 校验课程信息
            basicCourseService.verifyCourseInfo(schoolId, courseCode, courseName, userId);

            // 解析时间
            Map<String, Object> timeMap = ConvertUtil.analyzeStartAndEndTime(examDate, examTime);
            String examStartTime = String.valueOf(timeMap.get("startTime"));
            String examEndTime = String.valueOf(timeMap.get("endTime"));

            Map<String, Object> dataMap = new HashMap<>();
            dataMap.put("studentCode", studentCode);
            dataMap.put("studentName", studentName);
            dataMap.put("courseCode", courseCode);
            dataMap.put("courseName", courseName);
            dataMap.put("examPlace", examPlace);
            dataMap.put("examRoom", examRoom);
            dataMap.put("examStartTime", examStartTime);
            dataMap.put("examEndTime", examEndTime);
            dataMap.put("paperNumber", paperNumber);
            dataMap.put("secondaryFieldList", secondaryFieldList);
            dataMap.put("schoolId", schoolId);
            dataMap.put("printPlanId", printPlanId);
            dataMap.put("printPlanName", printPlanName);
            dataList.add(dataMap);
        }
        System.out.println("dataList = " + JSON.toJSONString(dataList));

        // 删除印刷计划下的考务数据
        examDetailService.deleteExaminationData(printPlanId);
        // 组装exam_detail数据
        examDetailService.disposeExamDetailByExaminationExcel(dataList, userId);
        // 组装exam_detail_course数据
        examDetailService.disposeExamDetailCourseByExaminationExcel(dataList, userId);
        // 组装exam_student数据
        examDetailService.disposeExamStudentByExaminationExcel(dataList, userId);

        List<Map<String, Object>> checkList = dataList.stream().flatMap(e -> {
            Map<String, Object> tmp = new HashMap<>();
            tmp.put("schoolId", e.get("schoolId"));
            tmp.put("courseCode", e.get("courseCode"));
            tmp.put("paperNumber", e.get("paperNumber"));
            return Stream.of(tmp);
        }).distinct().collect(Collectors.toList());
        for (Map<String, Object> stringObjectMap : checkList) {
            Long checkSchoolId = SystemConstant.convertIdToLong(String.valueOf(stringObjectMap.get("schoolId")));
            String checkCourseCode = String.valueOf(stringObjectMap.get("courseCode"));
            String checkPaperNumber = String.valueOf(stringObjectMap.get("paperNumber"));
            SysUser user = (SysUser) map.get(SystemConstant.USER);
            user.setSchoolId(schoolId);
            commonService.checkData(checkSchoolId, checkCourseCode, checkPaperNumber, user);
        }
        map.put("dataCount", dataList.size());
        return map;
    }
}
